#include "eeprom.h"
#include "serial.h"
#include "numbers.h"
#include "c++-alloc.h"

#include <propeller.h>
/*
Beta 2 Pins
*/

const int kPIN_EEPROM_SCL = 28;
const int kPIN_EEPROM_SDA = 29;

const int kPIN_USB_TX = 30; //Tx from the Propeller
const int kPIN_USB_RX = 31; //Rx to the Propeller

//const int kUSB_BAUD = 115200;
const int kUSB_BAUD = 460800;

Serial * debug;


const unsigned short kEepromUnitAddress = 0xFFFC;
const unsigned short kEepromBoardAddress = 0xFFF8;

const int kBoardAlpha = 0x0000000A;
const int kBoardBeta  = 0x0000000B;
const int kBoardBeta2 = 0x000000B2;
const int kBoardGamma = 0x00000004;


int GetBoardVersion(int oldVersion){
	
	char charBuffer[12];
	debug->Put("Please select the board version: \r\n");
	debug->Put(" * - 0x%08x (current version)\r\n", oldVersion);
	debug->Put(" a - Alpha\r\n");
	debug->Put(" b - Beta\r\n");
	debug->Put(" c - Beta 2\r\n");
	debug->Put(" d - Gamma\r\n");
	debug->Put(" e - Other\r\n");
	debug->Put(">>> ");
	debug->GetFlush();
	char choice = debug->Get();	
	debug->Put("%c\r\n", choice);
	int boardVersion;
	
	if(choice ==  'a'){ boardVersion = kBoardAlpha;}
	else if(choice == 'b'){ boardVersion = kBoardBeta;}
	else if(choice == 'c'){ boardVersion = kBoardBeta2;}
	else if(choice == 'd'){ boardVersion = kBoardGamma;}
	else if(choice == '*'){ boardVersion = oldVersion;}
	else{ 
		debug->Put("Your choice '0x%08x' isn't supported right now. Keeping current version.\r\n", choice);
		boardVersion = oldVersion;
	}
	
	return boardVersion;
}

int GetUnitNumber(int oldNumber){
	
	debug->Put("Please enter the unit number or * to keep current number, then <enter>:\r\n>>> ");
	debug->GetFlush();
	
	char buffer[13];
	for(int i = 0;;++i){
		buffer[i] = debug->Get();
		debug->Put(buffer[i]); //echo
		if(buffer[i] == '\r'){
			buffer[i] = 0;
			debug->Put('\n');
			break;
		}
	}
	int unitNumber;
	if(buffer[0] == '*'){
		unitNumber = oldNumber;
	}else{
	 	unitNumber = Numbers::Dec(buffer);
	}

	return unitNumber;
		

}


int main(void){
	
	debug = new Serial;
	debug->Start(kPIN_USB_RX, kPIN_USB_TX, kUSB_BAUD);	
	debug->Put("Configuration Utility!\r\n");
	
	Eeprom eeprom;
	
	char temp[4];
	
	int boardVersion = 0;
	int unitNumber = 0;
	

	for(;;){
		

		unitNumber = eeprom.Get(kEepromUnitAddress, 4);
		boardVersion = eeprom.Get(kEepromBoardAddress, 4);

	
		debug->Put("\r\n--------------------------\r\n\r\n");
		debug->Put("Current unit number:   %d\r\n", unitNumber);		
		debug->Put("Current board version: 0x%x\r\n", boardVersion);
		debug->Put("\r\n--------------------------\r\n\r\n");
	
		unitNumber = GetUnitNumber(unitNumber);
		eeprom.Put(kEepromUnitAddress, unitNumber, 4);
		
		boardVersion = GetBoardVersion(boardVersion);
		eeprom.Put(kEepromBoardAddress, boardVersion, 4);
		

	}
	

	delete debug;
	

}

































