//
// automatically generated by spin2cpp v1.02 on Sat Dec 29 22:32:12 2012
// spin2cpp.linux serial.spin 
//

//TODO (SRLM): check for remaining int32_t, etc.
//TODO (SRLM): Should return values really be -1?

//TODO(SRLM): I think the functions with buffer in them should have "volatile"
//in the function declaration (since the buffer can be modified by PASM).

/**

Based on Fast Full-Duplex Serial 1 (FFDS1) version 0.9 by Jonathan Dummer
(lonesock). C++ Port done by SRLM.

Serial provides a fast and stable serial interface using a single cog.

Max baudrate = clkfreq / (86 * 2)

    Clock  | MaxBaud | Standard
    -------+---------+---------
    96 MHz | 558_139 | 500_000    <- 6MHz XTAL at 16x PLL
    80 MHz | 465_116 | 460_800    <- 5MHz XTAL at 16x PLL (most common)
    12 MHz |  69_767 |  57_600    <- approx RCFAST
    20 kHz |     116 | hah hah    <- approx RCSLOW

Bit period is calculated to the nearest 2 clocks. So, the bit period should be
within 1 clock, or 12.5 ns at 80 MHz.

@author SRLM
@date   2013-01-21
*/


#ifndef SRLM_PROPGCC_SERIAL_H_
#define SRLM_PROPGCC_SERIAL_H_

#include <stdint.h>
#include <cstdarg>
#include "numbers.h"

class Serial {
public:

/**
Size of the RX buffer in bytes. No restrictions on size (well, Hub RAM limits
[8^) Does not need to be a power of 2. There is no TX buffer, Serial sends
directly from Hub RAM.

@warning: Do not change (it's hardcoded elsewhere)
@todo(SRLM): allow for changing kBufferLength.
*/
static const int kBufferLength = 256;

/*
Starts the Serial PASM engine running in a new cog.
@param rxpin the pin [0..31] for incoming data.
@param txpin the pin [0..31] for outgoing data. Must not equal rxpin. Use -1 to
             disable TX.
@param rate  the initial baud rate in bits per second.
@return  Returns (-1) if the cog started OK
*/
int Start(int rxpin, int txpin, int rate);


/** Stops the Serial PASM engine, if it is running.
*/
void Stop(void);
  
/** Does a live update of the baud rate in the Serial engine.
@todo(SRLM): Make this return a bool instead of an int...
@param rate desired baud rate of the Serial engine.
@return     -1 if the baud rate was achieved
*/
int SetBaud(int rate);

/** Does a live update of the baud rate in the Serial engine.

Example:

    got_desired_rate = SetBaudClock( 9600, CLKFREQ )
    got_desired_rate = SetBaudClock( 9600, actual_sys_clock_freq )

@param rate     desired baud rate of the Serial engine.
@param sysclock the system clock (use CLKFREQ), or provide your own (useful when
                using RCFAST).
@return         -1 if the baud rate was achieved
*/
  int	SetBaudClock(int rate, int sysclock);
  

/** Sends down a single character.

@param character the byte to send.
*/
void Put(char character);

/** printf function-alike.

This function is based on the requirements found on this page:
http://www.cplusplus.com/reference/cstdio/printf/

@warning This function does not implement the full printf functionality.

Formatters must follow the following prototype:
%[flags][width]specifier

The following flags are supported

- 0 (only with x,X,b,B). Sets padding to the 0 character instead of space.

The following 

This funciton supports the following specifiers:

- d or i : signed decimal integer. The width specifier will pad with spaces if
           necessary.
- x or X : hexdecimal interger. All caps (with either). The width specifier will
           pad with spaces (or 0s if the 0 flag is set) if necessary
- c      : output a single character.
- s      : output a string of characters, 0 terminated.

Each specifier must have a matching optional argument of the correct type.

@param format the string to send, optionally with specifiers.
@param ...    additional arguments. Depending on the format string, the function
              may expect a sequence of additional arguments.
@returns      on success, the total number of characters written. On error, a
              negative number is returned.
*/
int Put(const char * format, ...);


/** Get a buffer of characters

@param buffer  The buffer to store the received characters in.
@param length  The maximum number of characters to read (may be less than @a
               length if @a timeout is enabled.
@param timeout The maximum amount of time to wait for _each_ character, _not_
               the total time of the function.
@returns the number of characters read. If less than @a length, then a timeout
         occurred.
*/
int Get(char * buffer, int length, int timeout=-1);

/** Flushes receive buffer.
*/
void GetFlush(void);

/**
Waits for a byte to be received or a timeout to occur.
 ms : the number of milliseconds to wait for an incoming byte
 returns -1 if no byte received, $00..$FF if byte
 
@warning This function may block indefinately if timeout is set to a negative
         value, and no data is received.
 
@param timeout the number of milliseconds to wait. Defaults to forever wait (negative timeout)
*/
int Get(int timeout = -1);


private:


  

/**
Half period must be at least this, otherwise the cog will
sleep for a whole counter cycle (2^32 / clkfreq seconds).
*/
static const int kMinimumHalfPeriod = 86;
  
/**not used directly, but handy to know*/
static const int kXon = 17;
static const int kXoff = 19;
  
/**
The binary version of the PASM driver. Must be binary so that we can do garbage
collection with the linker.
*/  
static uint8_t dat[];

/*
Checks if byte is waiting in the buffer, but doesn't wait.
 returns -1 if no byte received, $00..$FF if byte
*/
  int	CheckBuffer(void);

  int32_t	volatile write_buf_ptr_;
  int32_t	volatile send_temp_;
  int32_t	volatile half_bit_period_;
  uint16_t	volatile rx_head_;
  uint16_t	volatile rx_tail_;
  uint8_t	volatile rx_buffer_[kBufferLength];
  uint8_t	cog_;
};


/*
Sends a string over the serial line.
 string_ptr : a pointer to a 0-terminated string in Hub RAM
 Does not transmit the 0 termination.
*/
//  void	PutS(const char * string_ptr);



#endif //SRLM_PROPGCC_SERIAL_H_
